import 'package:animated_icon_button/animated_icon_button.dart';
import 'package:buttons_swiper/src/swiperController.dart';
import 'package:flutter/material.dart';
import 'package:card_swiper/card_swiper.dart';
import 'package:get/get.dart';

class swiper extends StatelessWidget {
  final swiperControll = Get.put(SwiperGetxController());
  final Alignment alignement;
  final double height;
  final Color circleBorderColor;
  double circleBorderWidth;
  final ValueChanged<int> onTap;
  List<Icon> firstIconsList;
  List<Icon> secondIconsList;
  bool loop;
  double fade;
  int startIndex;
  double viewportFraction;
  double scale;

  swiper({
    required this.alignement,
    required this.height,
    this.circleBorderWidth = 5,
    required this.circleBorderColor,
    required this.onTap,
    required this.firstIconsList,
    required this.secondIconsList,
    this.loop = false,
    this.fade = 0.01,
    this.startIndex = 0,
    this.viewportFraction = 0.3,
    this.scale = 0.01,
  }) {
    swiperControll.borderWidth.value = this.circleBorderWidth;
    swiperControll.index = this.startIndex;
    if (firstIconsList.length != secondIconsList.length) {
      throw "Please Provide Equal Icon Arrays";
    }
  }

  @override
  Widget build(BuildContext context) {
    return Align(
      alignment: alignement,
      child: Container(
        height: height,
        child: Stack(
          children: [
            Obx(
              () => Center(
                child: AnimatedContainer(
                  width: swiperControll.width.value,
                  height: swiperControll.hight.value,
                  decoration: BoxDecoration(
                      border: Border.all(
                          color: circleBorderColor,
                          width: swiperControll.borderWidth.value),
                      shape: BoxShape.circle),
                  duration: Duration(milliseconds: 100),
                  curve: Curves.bounceIn,
                ),
              ),
            ),
            Swiper(
              controller: swiperControll.controller,
              itemBuilder: (BuildContext context, int index) {
                return AnimatedButton(
                  index: index,
                  onTap: (Index) async {
                    if (index == swiperControll.index) {
                      await swiperControll.animate(Index);
                    }
                    onTap.call(Index);
                  },
                  icon: firstIconsList[index],
                  secondIcon: secondIconsList[index],
                );
              },
              itemCount: firstIconsList.length,
              onIndexChanged: (index) {
                swiperControll.index = index;
              },
              loop: loop,
              fade: fade,
              index: startIndex,
              pagination: SwiperPagination(
                  alignment: Alignment.bottomCenter,
                  builder: SwiperPagination.rect),
              viewportFraction: viewportFraction,
              scale: scale,
            ),
          ],
        ),
      ),
    );
  }
}

class AnimatedButton extends StatefulWidget {
  Icon icon;
  Icon secondIcon;
  final ValueChanged<int> onTap;
  final int index;

  AnimatedButton(
      {required Icon this.icon,
      required Icon this.secondIcon,
      required this.onTap,
      required this.index}) {}

  @override
  _AnimatedButtonState createState() => _AnimatedButtonState(
      onTap: onTap, icon: icon, secondIcon: secondIcon, index: index);
}

class _AnimatedButtonState extends State<AnimatedButton>
    with SingleTickerProviderStateMixin {
  Icon icon;
  Icon secondIcon;
  final ValueChanged<int> onTap;
  final int index;

  _AnimatedButtonState({
    required Icon this.icon,
    required Icon this.secondIcon,
    required this.onTap,
    required this.index,
  }) {}
  final swiperControll = Get.find<SwiperGetxController>();
  late AnimationController animationController;

  @override
  void initState() {
    animationController = new AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 100),
      reverseDuration: Duration(milliseconds: 50),
    );
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedIconButton(
      animationController: animationController,
      onPressed: () async {
        await Future.delayed(const Duration(milliseconds: 300), () {});
        animationController.reverse();
        swiperControll.controller.move(index);
        onTap.call(index);
      },
      padding: EdgeInsets.all(0.0),
      alignment: Alignment.center,
      size: 80,
      splashRadius: 5.0,
      icons: [
        AnimatedIconItem(
          icon: icon,
        ),
        AnimatedIconItem(icon: secondIcon),
      ],
    );
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }
}
